//---------------------------------------------------------------------------
// Copyright (C) 1998 Dallas Semiconductor Corporation, All Rights Reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
// OTHER DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of Dallas Semiconductor 
// shall not be used except as stated in the Dallas Semiconductor 
// Branding Policy. 
//---------------------------------------------------------------------------
//
//  DebugPkt.h - Debug functions for MicroLan repeater packet
//               communication.
//
//  Version: 1.00
//

// includes
#include <stdio.h>
#include "mlanpkt.h"

// function prototypes
int Printpacket(uchar *, int);

// command strings enumeration from 0x80
char single_zstr[][30] = 
     { { "CMD_ML_RESET" }, 
       { "CMD_ML_SEARCH" },
       { "CMD_ML_ACCESS" },
       { "CMD_ML_OVERDRIVE_ACCESS" },
       { "CMD_RESET" }, 
       { "CMD_GETBUF" }, 
       { "CMD_ERROR" }, 
       { "" } };

// data operations, enumerate from 0x00
char multi_zstr[][30] =
     { { "DATA_ID" }, 
       { "DATA_SEARCH_STATE" }, 
       { "DATA_SEARCH_CMD" },
       { "DATA_MODE" }, 
       { "DATA_CAPABILITY" }, 
       { "DATA_OUTBOUND_MAX" }, 
       { "DATA_INBOUND_MAX" }, 
       { "DATA_PROTOCOL" }, 
       { "DATA_VENDOR" }, 
       { "CMD_ML_BIT" }, 
       { "CMD_ML_DATA" }, 
       { "CMD_DELAY" }, 
       { "" } };
       
// return codes strings enumeration from 0x00
char return_zstr[][30] = 
     { { "RET_SUCCESS" },
       { "RET_END_SEARCH" },
       { "RET_BUSY" },
       { "RET_ERROR" },
       { "RET_NO_DEVICE" },
       { "RET_ML_SHORTED" },
       { "RET_OUTBOUND_OVERRUN" }, 
       { "RET_INBOUND_OVERRUN" }, 
       { "RET_REG_OVERRUN" }, 
       { "RET_END_OF_INBOUND" }, 
       { "RET_READ_ONLY" }, 
       { "RET_WRITE_ONLY" },
       { "RET_CMD_UNKNOWN" }, 
       { "" } };

// mode bit flags strings bits 0, 1, 2, and 3
char mode_zstr[][30] = 
     { { "MODE_OVERDRIVE" },
       { "MODE_STRONG5" },
       { "MODE_PROGRAM" },
       { "MODE_BREAK" }, 
       { "" } };
       

//--------------------------------------------------------------------------
// Print the MicroLan repeater packet in debug format
//
int PrintPacket(uchar *packet, int packet_type)
{
   int pk_cnt = 0,i,start;
   uchar in_cmd, in_len, packet_length;

   // get the length byte
   packet_length = packet[pk_cnt++];

   // print type header for packet
   if (packet_type == INBOUND)
      printf("\ninbound (from host): (%d bytes)\n",packet_length + 1);
   else
      printf("outbound (to host): (%d bytes)\n",packet_length + 1);
   // print the raw packet
   printf("    Raw packet: ");
   for (i = 0; i <= packet[0]; i++)
   {
      printf("%02X ",packet[i]);
      if ((((i+1) % 16) == 0) && (i != 0))
         printf("\n                ");
   }

   // loop through the entire packet and print interpreted debug output
   printf("\n    Interpreted:\n");
   while (pk_cnt < (packet_length + 1))
   {
      // get a byte, could be data or command
      in_cmd = packet[pk_cnt++];

      // command
      if (in_cmd & SINGLE_BYTE_CMD)
      {
         // check for unknown command
         if (in_cmd > MaxSingleCommand)
            printf("    UNKNOWN COMMAND (%02X)",in_cmd);
         else
            // print the command
            printf("    %s",single_zstr[in_cmd - 0x80]);

         // check for return code
         if (packet_type == OUTBOUND)
         {
            // get a byte, return code
            in_cmd = packet[pk_cnt++];
            
            // check for unknown return code
            if (in_cmd > MaxRetCode)
               printf("    UNKNOWN RETURN CODE (%02X)",in_cmd);
            else
               // print the return code
               printf(" = %s",return_zstr[in_cmd]);
         }
      }
      // data
      else
      {
         // check for unknown data
         if (in_cmd > MaxMultiCommand)
            printf("    UNKNOWN DATA (%02X)",in_cmd);
         else
            // print the data operator
            printf("    %s",multi_zstr[in_cmd]);

         // get the length byte
         in_len = packet[pk_cnt++];
      
         // check for read inbound
         if (in_len == 0)
            printf(" <READ>");
         // must be result of read outbound
         else
         {
            start = 0;
            if (packet_type == INBOUND)
            {
               if (in_cmd == CMD_ML_DATA)
               {
                  i = packet[pk_cnt++];
                  printf(" <BLOCK(%d writes, %d slave reads)",in_len - 1,
                         i - in_len + 1);
                  start = 1;   
               }
               else
                  printf(" <WRITE(%d bytes)",in_len);
            }
            else
            {
               printf(" <(%d bytes)",in_len);
            }

            // print as zstr if version
            if ((in_cmd == DATA_PROTOCOL) || (in_cmd == DATA_VENDOR))
            {
               printf(" \"");
               for (i = 0; i < in_len; i++)
                  printf("%c",packet[pk_cnt++]);
               printf("\"");
            }
            // print as mode flags 
            else if ((in_cmd == DATA_MODE) || (in_cmd == DATA_CAPABILITY))
            {
               printf(" [%02X]",packet[pk_cnt]);
               if (packet[pk_cnt] & MODE_OVERDRIVE)
                  printf(" %s",mode_zstr[0]);
               if (packet[pk_cnt] & MODE_STRONG5)
                  printf(" %s",mode_zstr[1]);
               if (packet[pk_cnt] & MODE_PROGRAM)
                  printf(" %s",mode_zstr[2]);
               if (packet[pk_cnt] & MODE_BREAK)
                  printf(" %s",mode_zstr[3]);
               pk_cnt++;
            }
            // print as raw hex bytes
            else
            {
               for (i = start; i < in_len; i++)
               {
                  printf(" %02X",packet[pk_cnt++]);
                  // try to wrap 
                  if ((((i + 1) % 16) == 0) && (i != 0) && 
                      (i != (in_len - 1)))
                     printf("\n                          ");
               }
            }
            printf(">");
         }
      }
      
      printf("\n");
   }

   return TRUE;
}