//---------------------------------------------------------------------------
// Copyright (C) 1998 Dallas Semiconductor Corporation, All Rights Reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
// OTHER DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of Dallas Semiconductor 
// shall not be used except as stated in the Dallas Semiconductor 
// Branding Policy. 
//---------------------------------------------------------------------------
//
//  TranNetU.C - MicroLAN Transport and Network functions for repeater 
//               packet processing using DS2480 serial communication.
//
//  Version: 1.00
//

#include "mlanpkt.h"
#include "DS2480.h"

// external Link MicroLAN functions
extern uchar MLanDelay(uchar);
extern uchar MLanTouchReset(void);
extern uchar MLanTouchBit(uchar);
extern uchar MLanTouchByte(uchar);
extern void UpdateMLanMode(uchar);
extern uchar MLanLevel(uchar);

// packet processing functions
extern uchar SingleResult(uchar, uchar); 
extern int   CheckOutboundSpace(uchar);

// external COM functions required
extern void FlushCOM(void);
extern int  WriteCOM(int, uchar *);
extern int  ReadCOM(int, uchar *);

// external DS2480 functions
extern int DS2480Detect(void);

// local Transport and Network MicroLAN functions
uchar MLanSearch(void);
uchar MLanAccess(void);
uchar MLanOverdriveAccess(void);
uchar MLanBitBlock(uchar, uchar *);
uchar MLanByteBlock(uchar, uchar *);
uchar dowcrc(uchar);

// local static functions       
static uchar dowcrc(uchar);
static int bitacc(int, int, int, uchar *);

// global variables for this module to hold search state information
static uchar DOWCRC;

// external globals
extern int UMode;
extern int UBaud;
extern int USpeed;

// constants
#define WRITE_FUNCTION 0
#define READ_FUNCTION  1


//----------------------------------------------------------------------
//  Write and read the block of bit commands.
//
//  'data_length' - number of data bytes provided that represent
//                  the number of bit operations to do
//  '*data_bytes' - pointer to the begining of data bytes 
//                  (points into inbound buffer)
//
// Returns:  RET_SUCCESS: bit operations successfully done and
//                        the results have been placed in the 
//                        outbound frame.
//           RET_OUTBOUND_OVERRUN: error, there was not enough 
//                        room in outbound frame.
//           RET_ERROR:   error communicating with DS2480
//
//  *Note: data_length must be less than 16 bytes  
//
uchar MLanBitBlock(uchar data_length, uchar *data_bytes)
{
   uchar readbuffer[20],sendpacket[20];
   int i, sendlen=0;

   // 0 length not valid
   if (data_length == 0)
      return SingleResult(CMD_ERROR,RET_WRITE_ONLY); 

   // make sure normal level
   MLanLevel(MODE_NORMAL);

   // check size required in outbound
   if (CheckOutboundSpace((uchar)(data_length + 2)))
   {
      // add command to outbound
      outbound[++outbound[0]] = CMD_ML_BIT;

      // add the length to outbound
      outbound[++outbound[0]] = data_length;

      // create a packet to go to DS2480 to do bit operations
      // check if correct mode 
      if (UMode != MODSEL_COMMAND)
      {
         UMode = MODSEL_COMMAND;
         sendpacket[sendlen++] = MODE_COMMAND;
      }

      // construct the command
      for (i = 0; i < data_length; i++)
      {      
         sendpacket[sendlen] = (data_bytes[i] != 0) ? BITPOL_ONE : BITPOL_ZERO;
         sendpacket[sendlen++] |= CMD_COMM | FUNCTSEL_BIT | USpeed;
      }

      // flush the buffers
      FlushCOM();

      // send the packet 
      if (WriteCOM(sendlen,sendpacket)) 
      {
         // read back the response 
         if (ReadCOM(data_length,readbuffer) == data_length)
         {
            // interpret the response 
            for (i = 0; i < data_length; i++)
            {      
               if (((readbuffer[i] & 0xE0) == 0x80) &&
                   ((readbuffer[i] & RB_BIT_MASK) == RB_BIT_ONE))
                  outbound[++outbound[0]] = 1;
               else
                  outbound[++outbound[0]] = 0;
            }
            return RET_SUCCESS;
         }
      }

      // an error occured so re-sync with DS2480
      DS2480Detect();

      // adjust the outbound to have a an error
      outbound[outbound[0] - 1] = CMD_ERROR;
      outbound[outbound[0]] = RET_ERROR;

      return RET_ERROR;
   }
   else
      return RET_OUTBOUND_OVERRUN;
}


//----------------------------------------------------------------------
//  Write and read a block of bytes to the MicroLan.  The first
//  data byte is a length byte that represents the total number
//  of data byte transactions that are to be done.  If this
//  exceeds the number of data bytes available then the remaining
//  byte communications are slave reads represented by the
//  master writing 0xFF and reading the result. 
//
//  'data_length' - number of data bytes provided.  The first
//                  data byte is a length byte.
//  '*data_bytes' - pointer to the begining of data bytes 
//                  (points into inbound buffer)
//
// Returns:  RET_SUCCESS: MicroLan communication successfully done and
//                        the results have been placed in the 
//                        outbound frame.
//           RET_OUTBOUND_OVERRUN: error, there was not enough 
//                        room in outbound frame.
//           RET_ERROR:   error communicating with DS2480
//
//  *Note: data_length must be less than 45 bytes  
//
uchar MLanByteBlock(uchar data_length, uchar *data_bytes)
{
   uchar sendpacket[50];
   int sendlen=0;
   int i;
   uchar wr_len;
   
   // 0 length not valid
   if (data_length == 0)
      return SingleResult(CMD_ERROR,RET_WRITE_ONLY); 

   // make sure normal level
   MLanLevel(MODE_NORMAL);

   // get the write length byte
   wr_len = data_bytes[0];

   // check size required in outbound
   if (CheckOutboundSpace((uchar)(wr_len + 2)))
   {
      // add command to outbound
      outbound[++outbound[0]] = CMD_ML_DATA;

      // add the lenght
      outbound[++outbound[0]] = wr_len;

      // create a packet to go to DS2480 to do byte operations
      // check if correct mode 
      if (UMode != MODSEL_DATA)
      {
         UMode = MODSEL_DATA;
         sendpacket[sendlen++] = MODE_DATA;
      }

      // add the data itself
      for (i = 0; i < wr_len; i++)
      {
         // check if sending data or read 0xFF's
         if (i < (data_length - 1))
         {
            // add the byte to send
            sendpacket[sendlen++] = data_bytes[i + 1];
            // check for duplication of data that looks like COMMAND mode 
            if (data_bytes[i + 1] == MODE_COMMAND) 
               sendpacket[sendlen++] = data_bytes[i + 1];
         }
         else
            sendpacket[sendlen++] = 0xFF;
      }

      // flush the buffers
      FlushCOM();

      // send the packet 
      if (WriteCOM(sendlen,sendpacket)) 
      {
         // read back the 1 byte response 
         if (ReadCOM(wr_len,&outbound[outbound[0] + 1]) == wr_len)
         {
            // adjust the outbound length
            outbound[0] += wr_len;
            // return success 
            return RET_SUCCESS;
         }
      }

      // an error occured so re-sync with DS2480
      DS2480Detect();

      // adjust the outbound to have a an error
      outbound[outbound[0] - 1] = CMD_ERROR;
      outbound[outbound[0]] = RET_ERROR;

      return RET_ERROR;
   }
   else
      return RET_OUTBOUND_OVERRUN;
}


//----------------------------------------------------------------------
// Perform a MATCH command to select the 1-Wire device with the
// address in the ID data register.
//
// Returns:  RET_SUCCESS: If the device is present on the MicroLAN
//                        then the device is selected.
//           RET_NO_DEVICE: Error, no devices present on the MicroLAN
//           RET_ML_SHORTED: Error, MicroLAN shorted, communication not
//                        possible.
//
//  *Note: This function could be converted to send DS2480
//         commands in one packet.  
//
uchar MLanAccess(void)
{
   int i;
   uchar return_code;

   // call the MicroLan reset function 
   return_code = MLanTouchReset();
   if (return_code == RET_SUCCESS)
   {
      // send the match command 0x55
      if (MLanTouchByte(0x55) == 0x55)
      {
         // loop to send the DATA_ID
         for (i = 0; i < 8; i++)
         {
            if (MLanTouchByte(id[i]) != id[i])
               return RET_ML_SHORTED;
         }
      }
      else
         return RET_ML_SHORTED;
   }
   else
      return RET_NO_DEVICE;
   
   return RET_SUCCESS;
}


//----------------------------------------------------------------------
// Perform a MATCH command to select the 1-Wire device with the
// address in the ID data register.
//
// Returns:  RET_SUCCESS: If the device is present on the MicroLAN
//                        then the device is selected.
//           RET_ERROR:   Error, this repeater does not support overdrive
//           RET_NO_DEVICE: Error, no devices present on the MicroLAN
//           RET_ML_SHORTED: Error, MicroLAN shorted, communication not
//                        possible.
//
//  *Note: This function could be converted to send DS2480
//         commands in one packet.  
//
uchar MLanOverdriveAccess(void)
{
   int i;
   uchar return_code;

   // check for overdrive capability
   if (!(capability[0] & MODE_OVERDRIVE))
      return RET_ERROR;

   // make sure normal level
   MLanLevel(MODE_NORMAL);

   // force repeater to normal communication speed
   mode[0] = MODE_NORMAL;
   UpdateMLanMode(mode[0]);

   // call the MicroLan reset function 
   return_code = MLanTouchReset();
   if (return_code == RET_SUCCESS)
   {
      // send the match command 0x69
      if (MLanTouchByte(0x69) == 0x69)
      {
         // switch repeater to overdrive communication speed
         mode[0] = MODE_OVERDRIVE;
         UpdateMLanMode(mode[0]);

         // loop to send the DATA_ID in overdrive
         for (i = 0; i < 8; i++)
         {
            if (MLanTouchByte(id[i]) != id[i])
               return RET_ML_SHORTED;
         }
      }
      else
         return RET_ML_SHORTED;
   }
   else
      return RET_NO_DEVICE;
   
   return RET_SUCCESS;
}


//--------------------------------------------------------------------------
// The 'MLanSearch' function does a general search.  This function
// continues from the previous search state. The search state
// can be reset by setting the search state variables to 0.  For
// a normal search, the CMD_ML_RESET command must be called before
// this search function.
//
// Returns:  RET_SUCCESS: when a 1-Wire device was found and it's 
//                        ROM number placed in the global ROM
//           RET_END_SEARCH: when no new device was found.  Either 
//                        the last search was the last device or 
//                        there are no devices on the MicroLan.
// 
uchar MLanSearch(void)
{
   int i,last_zero,pos,id_bit_number;
   uchar TempROM[8];
   uchar readbuffer[20],sendpacket[40];
   uchar *LastDiscrepancy, *LastFamilyDiscrepancy, *ROM;
   int sendlen=0;

   // make sure normal level
   MLanLevel(MODE_NORMAL);

   // set pointers to data space variables
   LastDiscrepancy = &search_state[0];
   LastFamilyDiscrepancy = &search_state[1];
   ROM = &id[0];

   // if the last call was the last one 
   if (LastDeviceFlag)
   {
      // reset the search
      *LastDiscrepancy = 0;
      LastDeviceFlag = FALSE;
      *LastFamilyDiscrepancy = 0;          
      return RET_END_SEARCH;
   }

   // build the command stream
   // check if correct mode 
   if (UMode != MODSEL_DATA)
   {
      UMode = MODSEL_DATA;
      sendpacket[sendlen++] = MODE_DATA;
   }

   // search command
   sendpacket[sendlen++] = search_cmd[0];

   // change back to command mode
   UMode = MODSEL_COMMAND;
   sendpacket[sendlen++] = MODE_COMMAND;

   // search mode on
   sendpacket[sendlen++] = (uchar)(CMD_COMM | FUNCTSEL_SEARCHON | USpeed);

   // change back to data mode
   UMode = MODSEL_DATA;
   sendpacket[sendlen++] = MODE_DATA;

   // set the temp Last Descrep to none
   last_zero = 0xFF;  

   // add the 16 bytes of the search
   pos = sendlen;
   for (i = 0; i < 16; i++)
      sendpacket[sendlen++] = 0;

   // only modify bits if not the first search
   if (*LastDiscrepancy != 0xFF)
   {
      // set the bits in the added buffer
      for (id_bit_number = 0; id_bit_number < 64; id_bit_number++)
      {
         // before last discrepancy
         if (id_bit_number < (*LastDiscrepancy - 1)) 
               bitacc(WRITE_FUNCTION,
                   bitacc(READ_FUNCTION,0,id_bit_number,&ROM[0]), 
                   (short)(id_bit_number * 2 + 1), 
                   &sendpacket[pos]);
         // at last discrepancy
         else if (id_bit_number == (*LastDiscrepancy - 1)) 
                bitacc(WRITE_FUNCTION,1, 
                   (short)(id_bit_number * 2 + 1), 
                   &sendpacket[pos]);
         // after last discrepancy so leave zeros
      }
   }

   // change back to command mode
   UMode = MODSEL_COMMAND;
   sendpacket[sendlen++] = MODE_COMMAND;

   // search OFF
   sendpacket[sendlen++] = (uchar)(CMD_COMM | FUNCTSEL_SEARCHOFF | USpeed);

   // flush the buffers
   FlushCOM();

   // send the packet 
   if (WriteCOM(sendlen,sendpacket)) 
   {
      // read back the 1 byte response 
      if (ReadCOM(17,readbuffer) == 17)
      {
         // interpret the bit stream
         for (id_bit_number = 0; id_bit_number < 64; id_bit_number++)
         {
            // get the ROM bit
            bitacc(WRITE_FUNCTION,
                   bitacc(READ_FUNCTION,0,
                   (short)(id_bit_number * 2 + 1),&readbuffer[1]),
                   id_bit_number,&TempROM[0]);
            // check LastDiscrepancy
            if ((bitacc(READ_FUNCTION,0,
                   (short)(id_bit_number * 2),&readbuffer[1]) == 1) &&
                (bitacc(READ_FUNCTION,0,(short)(id_bit_number * 2 + 1),
                   &readbuffer[1]) == 0))
            {
               last_zero = id_bit_number + 1;  
               // check LastFamilyDiscrepancy
               if (id_bit_number < 8)
                  *LastFamilyDiscrepancy = id_bit_number + 1; 
            }
         }

         // do dowcrc
         DOWCRC = 0;
         for (i = 0; i < 8; i++)
            dowcrc(TempROM[i]);

         // check results 
         if ((DOWCRC != 0) || (*LastDiscrepancy == 63) || (TempROM[0] == 0))
         {
            // invalid search so reset 
            *LastDiscrepancy = 0;
            LastDeviceFlag = FALSE;
            *LastFamilyDiscrepancy = 0;          
            return RET_END_SEARCH;
         }
         // successful search
         else
         {
            // check for lastone
            if ((last_zero == *LastDiscrepancy) || (last_zero == 0xFF))
               LastDeviceFlag = TRUE;

            // copy the ROM to the id buffer
            for (i = 0; i < 8; i++)
               ROM[i] = TempROM[i];
         
            // set the count
            *LastDiscrepancy = last_zero;
            return RET_SUCCESS;
         }
      }
   }

   // an error occured so re-sync with DS2480
   DS2480Detect();

   // reset the search
   *LastDiscrepancy = 0;
   LastDeviceFlag = FALSE;
   *LastFamilyDiscrepancy = 0;          

   return RET_END_SEARCH;
}


//--------------------------------------------------------------------------
// Update the Dallas Semiconductor One Wire CRC (DOWCRC) from the global
// variable DOWCRC and the argument.  
//  
// 'x' - data to do the DOWCRC operation on.
//
// Returns: the updated DOWCRC.
//
uchar dscrc_table[] = {
        0, 94,188,226, 97, 63,221,131,194,156,126, 32,163,253, 31, 65,
      157,195, 33,127,252,162, 64, 30, 95,  1,227,189, 62, 96,130,220,
       35,125,159,193, 66, 28,254,160,225,191, 93,  3,128,222, 60, 98,
      190,224,  2, 92,223,129, 99, 61,124, 34,192,158, 29, 67,161,255,
       70, 24,250,164, 39,121,155,197,132,218, 56,102,229,187, 89,  7,
      219,133,103, 57,186,228,  6, 88, 25, 71,165,251,120, 38,196,154,
      101, 59,217,135,  4, 90,184,230,167,249, 27, 69,198,152,122, 36,
      248,166, 68, 26,153,199, 37,123, 58,100,134,216, 91,  5,231,185,
      140,210, 48,110,237,179, 81, 15, 78, 16,242,172, 47,113,147,205,
       17, 79,173,243,112, 46,204,146,211,141,111, 49,178,236, 14, 80,
      175,241, 19, 77,206,144,114, 44,109, 51,209,143, 12, 82,176,238,
       50,108,142,208, 83, 13,239,177,240,174, 76, 18,145,207, 45,115,
      202,148,118, 40,171,245, 23, 73,  8, 86,180,234,105, 55,213,139,
       87,  9,235,181, 54,104,138,212,149,203, 41,119,244,170, 72, 22,
      233,183, 85, 11,136,214, 52,106, 43,117,151,201, 74, 20,246,168,
      116, 42,200,150, 21, 75,169,247,182,232, 10, 84,215,137,107, 53};

uchar dowcrc(uchar x)
{
   DOWCRC = dscrc_table[DOWCRC ^ x];
   return DOWCRC;
}


//--------------------------------------------------------------------------
// Bit utility to read and write a bit in the buffer 'buf'.
// 
// 'op'    - operation flag, WRITE_FUNCTION (0), READ_FUNCTION (1)
// 'state' - value to set the indicated bit
// 'loc'   - bit location
// 'buf'   - array of bytes containing the bit to change
//
// Returns: 1   - if writing, write is a success
//          1/0 - if reading, value of bit location
//
int bitacc(int op, int state, int loc, uchar *buf)
{
     int nbyt,nbit;

     nbyt = (loc / 8);
     nbit = loc - (nbyt * 8);

     if (op == WRITE_FUNCTION)
     {
          if (state)
             buf[nbyt] |= (0x01 << nbit);
          else
             buf[nbyt] &= ~(0x01 << nbit);

          return 1;
     }
     else
          return ((buf[nbyt] >> nbit) & 0x01);
}
